@@ -1,13 +1,8 @@
-# coding: utf-8
-
 from django.core.management.base import BaseCommand
 from modeltranslation import settings as mt_settings
 from modeltranslation.utils import build_localized_fieldname
+from wagtail.core.models import Page
 from wagtail_modeltranslation.contextlib import use_language
-try:
-    from wagtail.core.models import Page
-except ImportError:
-    from wagtail.wagtailcore.models import Page
 
 
 class Command(BaseCommand):
