@@ -1,12 +1,8 @@
-# coding utf-8
-
 from modeltranslation.decorators import register
 from modeltranslation.translator import TranslationOptions
+from wagtail.core.models import Page
+
 from wagtail_modeltranslation import settings
-try:
-    from wagtail.core.models import Page
-except ImportError:
-    from wagtail.wagtailcore.models import Page
 
 
 @register(Page)
